package com.link.base.autorefresh.aop;

import com.link.base.autorefresh.annotation.AutoRefreshCache;
import com.link.base.autorefresh.thread.AutoRefreshCacheThread;
import com.link.core.cllog.LCLogger;
import com.link.core.thread.ThreadPoolExecutorSingleton;
import com.link.core.util.SpringUtil;
import java.lang.reflect.Method;
import java.util.concurrent.Executor;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.aop.support.AopUtils;

@Aspect
public class DelayRefreshCacheAspect {
   private static final String POINTCUT_STRING = "basicUpdate() || basicInsert() || basicDelete()";

   @Pointcut("execution(* com.link.core.basic.dao.mapper.BasicMapper.update(..))")
   public void basicUpdate() {
   }

   @Pointcut("execution(* com.link.core.basic.dao.mapper.BasicMapper.insert(..))")
   public void basicInsert() {
   }

   @Pointcut("execution(* com.link.core.basic.dao.mapper.BasicMapper.deleteById(..))")
   public void basicDelete() {
   }

   @AfterReturning(
      pointcut = "basicUpdate() || basicInsert() || basicDelete()",
      returning = "resultValue"
   )
   public void handleDataChangeAfterReturning(JoinPoint joinPoint, Object resultValue) {
      try {
         MethodSignature methodSignature = (MethodSignature)joinPoint.getSignature();
         Class mapperClass = (Class)AopUtils.getTargetClass(joinPoint.getTarget()).getGenericInterfaces()[0];
         AutoRefreshCache autoRefreshCache = (AutoRefreshCache)mapperClass.getAnnotation(AutoRefreshCache.class);
         if (null == autoRefreshCache) {
            return;
         }

         Integer effectiveRows = (Integer)resultValue;
         if (effectiveRows <= 0) {
            return;
         }

         String refreshFullClassPath = autoRefreshCache.invokeClassFullPath();
         String refreshMethodName = autoRefreshCache.invokeMethodName();
         boolean haveParamFlag = autoRefreshCache.haveParam();
         Object[] mapperMethodParams = joinPoint.getArgs();
         Object invokeClass = null;

         Exception invokeMethod;
         try {
            String beanName = refreshFullClassPath.substring(refreshFullClassPath.lastIndexOf(".") + 1, refreshFullClassPath.length());
            beanName = beanName.substring(0, 1).toLowerCase() + beanName.substring(1);
            invokeClass = SpringUtil.getBean(beanName);
         } catch (Exception var16) {
            invokeMethod = var16;
            invokeMethod.printStackTrace();
            LCLogger.error().withMessage(invokeMethod.getMessage()).flush();
            invokeClass = Class.forName(refreshFullClassPath).newInstance();
         }

         invokeMethod = null;
         Method invokeMethod;
         if (!haveParamFlag) {
            mapperMethodParams = null;
            invokeMethod = invokeClass.getClass().getDeclaredMethod(refreshMethodName, (Class[])null);
         } else {
            int length = mapperMethodParams.length;
            Class[] invokeMethodParamClass = new Class[length];

            for(int i = 0; i < length; ++i) {
               invokeMethodParamClass[i] = mapperMethodParams[i].getClass();
            }

            invokeMethod = invokeClass.getClass().getDeclaredMethod(refreshMethodName, invokeMethodParamClass);
         }

         Executor executor = ThreadPoolExecutorSingleton.getContextInstance();
         executor.execute(new AutoRefreshCacheThread(invokeClass, invokeMethod, mapperMethodParams));
      } catch (Exception var17) {
         Exception e = var17;
         e.printStackTrace();
      }

   }
}
